]> cloudbase.mooo.com Git - z180-stamp.git/blobdiff - avr/cli.c
Prompt
[z180-stamp.git] / avr / cli.c
index 046859c8f1648eadf8af4afcd19730de3242e4a3..922f9e1971ee7087ada1d0ab67488f358629e4ba 100644 (file)
--- a/avr/cli.c
+++ b/avr/cli.c
@@ -1,3 +1,17 @@
+/*
+ * (C) Copyright 2014-2016 Leo C. <erbl259-lmu@yahoo.de>
+ *
+ * (C) Copyright 2000
+ * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
+ *
+ * Add to readline cmdline-editing by
+ * (C) Copyright 2005
+ * JinHua Luo, GuangDong Linux Center, <luo.jinhua@gd-linux.com>
+ *
+ * SPDX-License-Identifier:    GPL-2.0
+ */
+
+#include "cli.h"
 #include "common.h"
 
 #include <string.h>
 #include "env.h"
 #include "cli_readline.h"
 #include "con-utils.h"
-#include "cli.h"
+
+
+/* FIXME: Quoting problems */
 
 #define DEBUG_PARSER   0       /* set to 1 to debug */
 
 #define debug_parser(fmt, args...)             \
        debug_cond(DEBUG_PARSER, fmt, ##args)
 
+
+static bool opt_xtrace;
+static bool opt_verbose;
+static int_least8_t command_level;
+
+static void cli_trace_cmd(int_fast8_t level, int argc, char *argv[])
+{
+       while (level-- > 0)
+               putchar('+');
+       for (int_fast8_t i = 0; i < argc; i++)
+               printf_P(PSTR(" %s"), argv[i]);
+       putchar('\n');
+}
+
+
+
+
 static int cli_parse_line(char *line, char *argv[])
 {
        uint_fast8_t state = 0;
@@ -28,12 +61,10 @@ static int cli_parse_line(char *line, char *argv[])
 
        debug_parser("%s: \"%s\"\n", __func__, line);
 
-       for (outp = inp = line, quote = '\0' ;
-                       nargs < CONFIG_SYS_MAXARGS && (c = *inp) != '\0';
-                       inp++) {
+       for (outp = inp = line, quote = '\0'; (c = *inp) != '\0'; inp++) {
 
                switch (state) {
-               case 0:
+               case 0:                                         /* before arg string, waiting for arg start */
                        if (isblank(c))
                                continue;
 
@@ -42,7 +73,7 @@ static int cli_parse_line(char *line, char *argv[])
                        state = 1;
                        /* fall thru */
 
-               case 1:
+               case 1:                                         /* in arg string, waiting for end of arg string */
                        if (c == '\\') {
                                ++state;
                                continue;
@@ -58,7 +89,7 @@ static int cli_parse_line(char *line, char *argv[])
                        }
                        break;
 
-               case 3:
+               case 3:                                         /* in quote */
                        if (c == '\\' && quote == '\"') {
                                ++state;
                                continue;
@@ -69,12 +100,17 @@ static int cli_parse_line(char *line, char *argv[])
                        }
                        break;
 
-               case 2:
+               case 2:                                         /* waiting for next char */
                case 4:
                        --state;
                        break;
 
                }
+
+               if (nargs > CONFIG_SYS_MAXARGS) {
+                       --nargs;
+                       break;
+               }
                *outp++ = c;
        }
 
@@ -158,7 +194,7 @@ char *process_macros(char *input, char *output)
                                if (c == '}') {
                                        /* Terminate variable name */
                                        *(inp-1) = '\0';
-                                       const char *envval = getenv(varname);
+                                       const char *envval = getenv_str(varname);
                                        *(inp-1) = '}';
                                        /* Copy into the line if it exists */
                                        if (envval != NULL)
@@ -191,7 +227,7 @@ char *process_macros(char *input, char *output)
  * WARNING:
  *
  * We must create a temporary copy of the command since the command we get
* may be the result from getenv(), which returns a pointer directly to
  * the environment data, which may change magicly when the command we run
  * creates or modifies environment variables (like "bootp" does).
  *
@@ -213,9 +249,20 @@ static int cli_run_command(const char *cmd, int flag)
        uint_fast8_t inquotes, repeatable = 1;
        int rc = 0;
 
+       opt_verbose = 0;
+       opt_xtrace = 0;
+       char *optenv = getenv_str(PSTR("cli"));
+       if (optenv) {
+               opt_verbose = (strstr_P(optenv, PSTR("verbose")) != NULL);
+               opt_xtrace = (strstr_P(optenv, PSTR("xtrace")) != NULL);
+       }
+
        debug_parser("[RUN_COMMAND] cmd[%p]=\"%s\"\n",
                        cmd, cmd ? cmd : "NULL");
 
+       if (opt_verbose)
+               printf_P(PSTR("%s\n"), cmd, cmd ? cmd : "");
+
        clear_ctrlc();          /* forget any previous Control C */
 
        if (!cmd || !*cmd)
@@ -225,6 +272,7 @@ static int cli_run_command(const char *cmd, int flag)
        if (!cmdbuf)
                return -1;      /* not enough memory */
 
+       ++command_level;
        str = cmdbuf;
 
        /* Process separators and check for invalid
@@ -239,25 +287,29 @@ static int cli_run_command(const char *cmd, int flag)
                 */
                for (inquotes = 0, sep = str; *sep; sep++) {
                        if ((*sep == '\'') &&
-                           (*(sep - 1) != '\\'))
+                           (sep != str) &&                                     /* past string start */
+                           (*(sep - 1) != '\\'))                       /* and NOT escaped */
                                inquotes = !inquotes;
 
                        if (!inquotes &&
-                           (*sep == ';' || *sep == '\n') &&    /* separator            */
-                           (sep != str) &&                     /* past string start    */
-                           (*(sep - 1) != '\\'))               /* and NOT escaped */
+                           (*sep == ';' || *sep == '\n'        /* separator */
+                                       || *sep == '#') &&                      /*   or start of comment */
+                           ((sep == str) ||                            /* string start */
+                           (*(sep - 1) != '\\')))                      /*   or NOT escaped */
                                break;
                }
 
-               /*
-                * Limit the token to data between separators
-                */
+               /* no more commands after unescaped '#' token */
+               if (*sep == '#')
+                       *sep = '\0';
+
+               /* Limit the token to data between separators */
                token = str;
                if (*sep) {
-                       str = sep + 1;  /* start of command for next pass */
+                       str = sep + 1;          /* start of command for next pass */
                        *sep = '\0';
                } else {
-                       str = sep;      /* no more commands for next pass */
+                       str = sep;                      /* no more commands for next pass */
                }
                debug_parser("token: \"%s\"\n", token);
 
@@ -271,8 +323,15 @@ static int cli_run_command(const char *cmd, int flag)
                        continue;
                }
 
-               if (cmd_process(flag, argc, argv, &repeatable) != CMD_RET_SUCCESS)
+               if (opt_xtrace)
+                       cli_trace_cmd(command_level, argc, argv);
+
+               rc = cmd_process(flag, argc, argv, &repeatable);
+               if (rc != CMD_RET_SUCCESS) {
+                       if (opt_verbose)
+                               printf_P(PSTR("Command failed, result=%d\n"), rc);
                        rc = -1;
+               }
 
                /* Did the user stop this? */
                if (had_ctrlc()) {
@@ -283,6 +342,7 @@ static int cli_run_command(const char *cmd, int flag)
 
        free(cmdbuf);
        free(finaltoken);
+       --command_level;
 
        return rc ? rc : repeatable;
 }
@@ -296,7 +356,7 @@ static int cli_run_command_list(const char *cmd)
 
 
 /*
- * Run a command using the selected parser.
+ * Run a command.
  *
  * @param cmd  Command to run
  * @param flag Execution flags (CMD_FLAG_...)
@@ -344,7 +404,7 @@ void cli_loop(void)
        int rc = 1;
 
        for (;;) {
-               len = cli_readline(PSTR(CONFIG_SYS_PROMPT));
+               len = cli_readline(lastcommand ? PSTR(CONFIG_SYS_PROMPT_REPEAT) : PSTR(CONFIG_SYS_PROMPT), 1);
 
                flag = 0;       /* assume no special flags for now */
                if (len > 0) {
@@ -353,9 +413,10 @@ void cli_loop(void)
                } else if (len == 0)
                        flag |= CMD_FLAG_REPEAT;
 
-               if (len == -1)
-                       my_puts_P(PSTR("<INTERRUPT>\n"));
-               else
+               if (len == -1) {
+                       if (opt_verbose)
+                               my_puts_P(PSTR("<INTERRUPT>\n"));
+               } else
                        rc = run_command_repeatable(lastcommand, flag);
 
                if (rc <= 0) {
@@ -365,27 +426,3 @@ void cli_loop(void)
                }
        }
 }
-
-
-command_ret_t do_run(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
-{
-       int i;
-       (void) cmdtp;
-
-       if (argc < 2)
-               return CMD_RET_USAGE;
-
-       for (i = 1; i < argc; ++i) {
-               char *arg;
-
-               arg = getenv(argv[i]);
-               if (arg == NULL) {
-                       printf_P(PSTR("## Error: \"%s\" not defined\n"), argv[i]);
-                       return CMD_RET_FAILURE;
-               }
-
-               if (run_command(arg, flag) != 0)
-                       return CMD_RET_FAILURE;
-       }
-       return CMD_RET_SUCCESS;
-}
@@ -191,7 +227,7 @@ char *process_macros(char *input, char *output)
  *
  * We must create a temporary copy of the command since the command we get
- * may be the result from getenv(), which returns a pointer directly to
+ * may be the result from getenv_str(), which returns a pointer directly to
  * the environment data, which may change magicly when the command we run
  * creates or modifies environment variables (like "bootp" does).